##############################################################################
#
# Copyright (c) 2003-2009 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Location support
"""
__docformat__ = 'restructuredtext'

from zope.interface import implementer
from zope.proxy import ProxyBase
from zope.proxy import getProxiedObject
from zope.proxy import non_overridable
from zope.proxy.decorator import DecoratorSpecificationDescriptor

from zope.location.interfaces import ILocation


@implementer(ILocation)
class Location:
    """Mix-in that implements ILocation.

    It provides the `__parent__` and `__name__` attributes.
    """

    __parent__ = None
    __name__ = None


def locate(obj, parent, name=None):
    """Update a location's coordinates."""
    obj.__parent__ = parent
    obj.__name__ = name


def located(obj, parent, name=None):
    """Ensure and return the location of an object.

    Updates the location's coordinates.
    """
    location = ILocation(obj)
    locate(location, parent, name)
    return location


def LocationIterator(object):
    """Iterate over an object and all of its parents."""
    while object is not None:
        yield object
        object = getattr(object, '__parent__', None)


def inside(l1, l2):
    """Test whether l1 is a successor of l2.

    l1 is a successor of l2 if l2 is in the chain of parents of l1 or l2
    is l1.

    """
    while l1 is not None:
        if l1 is l2:
            return True
        l1 = getattr(l1, '__parent__', None)
    return False


class ClassAndInstanceDescr:

    def __init__(self, *args):
        self.funcs = args

    def __get__(self, inst, cls):
        if inst is None:
            return self.funcs[1](cls)
        return self.funcs[0](inst)


@implementer(ILocation)
class LocationProxy(ProxyBase):
    """Location-object proxy

    This is a non-picklable proxy that can be put around objects that
    don't implement `ILocation`.
    """
    __slots__ = ('__parent__', '__name__')
    __safe_for_unpickling__ = True

    __doc__ = ClassAndInstanceDescr(
        lambda inst: getProxiedObject(inst).__doc__,
        lambda cls, __doc__=__doc__: __doc__,
    )

    def __new__(self, ob, container=None, name=None):
        return ProxyBase.__new__(self, ob)

    def __init__(self, ob, container=None, name=None):
        ProxyBase.__init__(self, ob)
        self.__parent__ = container
        self.__name__ = name

    def __getattribute__(self, name):
        if name in LocationProxy.__dict__:
            return object.__getattribute__(self, name)
        return ProxyBase.__getattribute__(self, name)

    def __setattr__(self, name, value):
        if name in self.__slots__ + getattr(ProxyBase, '__slots__', ()):
            # ('_wrapped', '__parent__', '__name__'):
            try:
                return object.__setattr__(self, name, value)
            except TypeError:  # pragma NO COVER C Optimization
                return ProxyBase.__setattr__(self, name, value)
        return ProxyBase.__setattr__(self, name, value)

    @non_overridable
    def __reduce__(self, proto=None):
        raise TypeError("Not picklable")
    __reduce_ex__ = __reduce__

    __providedBy__ = DecoratorSpecificationDescriptor()



##############################################################################
#
# Copyright (c) 2003-2009 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Classes to support implenting IContained
"""
__docformat__ = 'restructuredtext'

from zope.interface import implementer

from zope.location.interfaces import ILocationInfo
from zope.location.interfaces import IRoot
from zope.location.interfaces import ISite  # zope.component, if present


@implementer(ILocationInfo)
class LocationPhysicallyLocatable:
    """Provide location information for location objects
    """

    def __init__(self, context):
        self.context = context

    def getRoot(self):
        """See ILocationInfo.
        """
        context = self.context
        max = 9999
        while context is not None:
            if IRoot.providedBy(context):
                return context
            context = context.__parent__
            max -= 1
            if max < 1:
                raise TypeError("Maximum location depth exceeded, "
                                "probably due to a a location cycle.")

        raise TypeError("Not enough context to determine location root")

    def getPath(self):
        """See ILocationInfo.
        """
        path = []
        context = self.context
        max = 9999
        while context is not None:
            if IRoot.providedBy(context):
                if path:
                    path.append('')
                    path.reverse()
                    return '/'.join(path)
                return '/'
            path.append(context.__name__)
            context = context.__parent__
            max -= 1
            if max < 1:
                raise TypeError("Maximum location depth exceeded, "
                                "probably due to a a location cycle.")

        raise TypeError("Not enough context to determine location root")

    def getParent(self):
        """See ILocationInfo.
        """
        parent = getattr(self.context, '__parent__', None)
        if parent is not None:
            return parent

        raise TypeError('Not enough context information to get parent',
                        self.context)

    def getParents(self):
        """See ILocationInfo.
        """
        # XXX Merge this implementation with getPath. This was refactored
        # from zope.traversing.
        parents = []
        w = self.context
        while True:
            w = getattr(w, '__parent__', None)
            if w is None:
                break
            parents.append(w)

        if parents and IRoot.providedBy(parents[-1]):
            return parents

        raise TypeError("Not enough context information to get all parents")

    def getName(self):
        """See ILocationInfo
        """
        return self.context.__name__

    def getNearestSite(self):
        """See ILocationInfo
        """
        if ISite.providedBy(self.context):
            return self.context
        for parent in self.getParents():
            if ISite.providedBy(parent):
                return parent
        return self.getRoot()


@implementer(ILocationInfo)
class RootPhysicallyLocatable:
    """Provide location information for the root object

    This adapter is very simple, because there's no places to search
    for parents and nearest sites, so we are only working with context
    object, knowing that its the root object already.
    """

    def __init__(self, context):
        self.context = context

    def getRoot(self):
        """See ILocationInfo
        """
        return self.context

    def getPath(self):
        """See ILocationInfo
        """
        return '/'

    def getParent(self):
        """See ILocationInfo.
        """
        return None

    def getParents(self):
        """See ILocationInfo
        """
        return []

    def getName(self):
        """See ILocationInfo
        """
        return ''

    def getNearestSite(self):
        """See ILocationInfo
        """
        return self.context



##############################################################################
#
# Copyright (c) 2003-2009 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Location copying/pickling support
"""
__docformat__ = 'restructuredtext'

from zope.interface import implementer

from zope.location.location import inside


try:
    from zope.copy.interfaces import ICopyHook
    from zope.copy.interfaces import ResumeCopy
except ModuleNotFoundError:  # pragma: no cover
    raise NotImplementedError("zope.location.pickling is not supported "
                              "because zope.copy is not available")


@implementer(ICopyHook)
class LocationCopyHook:
    """Copy hook to preserve copying referenced objects that are not
    located inside object that's being copied.
    """

    def __init__(self, context):
        self.context = context

    def __call__(self, toplevel, register):
        if not inside(self.context, toplevel):
            return self.context
        raise ResumeCopy



##############################################################################
#
# Copyright (c) 2012 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
import unittest


class ConformsToILocation:

    def test_class_conforms_to_ILocation(self):
        from zope.interface.verify import verifyClass

        from zope.location.interfaces import ILocation
        verifyClass(ILocation, self._getTargetClass())

    def test_instance_conforms_to_ILocation(self):
        from zope.interface.verify import verifyObject

        from zope.location.interfaces import ILocation
        verifyObject(ILocation, self._makeOne())


class LocationTests(unittest.TestCase, ConformsToILocation):

    def _getTargetClass(self):
        from zope.location.location import Location
        return Location

    def _makeOne(self):
        return self._getTargetClass()()

    def test_ctor(self):
        loc = self._makeOne()
        self.assertEqual(loc.__parent__, None)
        self.assertEqual(loc.__name__, None)


class Test_locate(unittest.TestCase):

    def _callFUT(self, obj, *args, **kw):
        from zope.location.location import locate
        return locate(obj, *args, **kw)

    def test_wo_name(self):
        class Dummy:
            pass
        parent = Dummy()
        dummy = Dummy()
        self._callFUT(dummy, parent)
        self.assertIs(dummy.__parent__, parent)
        self.assertEqual(dummy.__name__, None)

    def test_w_name(self):
        class Dummy:
            pass
        parent = Dummy()
        dummy = Dummy()
        self._callFUT(dummy, parent, 'name')
        self.assertIs(dummy.__parent__, parent)
        self.assertEqual(dummy.__name__, 'name')


class Test_located(unittest.TestCase):

    def _callFUT(self, obj, *args, **kw):
        from zope.location.location import located
        return located(obj, *args, **kw)

    def test_wo_name_obj_implements_ILocation(self):
        from zope.interface import implementer

        from zope.location.interfaces import ILocation

        @implementer(ILocation)
        class Dummy:
            __parent__ = None
            __name__ = object()
        parent = Dummy()
        dummy = Dummy()
        self._callFUT(dummy, parent)
        self.assertIs(dummy.__parent__, parent)
        self.assertEqual(dummy.__name__, None)

    def test_w_name_adaptable_to_ILocation(self):
        from zope.interface.interface import adapter_hooks

        from zope.location.interfaces import ILocation
        _hooked = []

        def _hook(iface, obj):
            _hooked.append((iface, obj))
            return obj

        class Dummy:
            pass
        parent = Dummy()
        dummy = Dummy()
        before = adapter_hooks[:]
        adapter_hooks.insert(0, _hook)
        try:
            self._callFUT(dummy, parent, 'name')
        finally:
            adapter_hooks[:] = before
        self.assertIs(dummy.__parent__, parent)
        self.assertEqual(dummy.__name__, 'name')
        self.assertEqual(len(_hooked), 1)
        self.assertEqual(_hooked[0], (ILocation, dummy))

    def test_wo_name_not_adaptable_to_ILocation(self):
        class Dummy:
            __parent__ = None
            __name__ = 'before'
        parent = Dummy()
        dummy = Dummy()
        self.assertRaises(TypeError, self._callFUT, dummy, parent, 'name')
        self.assertEqual(dummy.__parent__, None)
        self.assertEqual(dummy.__name__, 'before')


class Test_LocationIterator(unittest.TestCase):

    def _callFUT(self, obj):
        from zope.location.location import LocationIterator
        return LocationIterator(obj)

    def test_w_None(self):
        self.assertEqual(list(self._callFUT(None)), [])

    def test_w_non_location_object(self):
        island = object()
        self.assertEqual(list(self._callFUT(island)), [island])

    def test_w_isolated_location_object(self):
        class Dummy:
            __parent__ = None
            __name__ = 'before'
        island = Dummy()
        self.assertEqual(list(self._callFUT(island)), [island])

    def test_w_nested_location_object(self):
        class Dummy:
            __parent__ = None
            __name__ = 'before'
        parent = Dummy()
        child = Dummy()
        child.__parent__ = parent
        grand = Dummy()
        grand.__parent__ = child
        self.assertEqual(list(self._callFUT(grand)), [grand, child, parent])


class Test_inside(unittest.TestCase):

    def _callFUT(self, i1, i2):
        from zope.location.location import inside
        return inside(i1, i2)

    def test_w_non_location_objects(self):
        island = object()
        atoll = object()
        self.assertTrue(self._callFUT(island, island))
        self.assertFalse(self._callFUT(island, atoll))
        self.assertFalse(self._callFUT(atoll, island))
        self.assertTrue(self._callFUT(atoll, atoll))

    def test_w_isolated_location_objects(self):
        class Dummy:
            __parent__ = None
            __name__ = 'before'
        island = Dummy()
        atoll = Dummy()
        self.assertTrue(self._callFUT(island, island))
        self.assertFalse(self._callFUT(island, atoll))
        self.assertFalse(self._callFUT(atoll, island))
        self.assertTrue(self._callFUT(atoll, atoll))

    def test_w_nested_location_object(self):
        class Dummy:
            __parent__ = None
            __name__ = 'before'
        parent = Dummy()
        child = Dummy()
        child.__parent__ = parent
        grand = Dummy()
        grand.__parent__ = child
        self.assertTrue(self._callFUT(child, parent))
        self.assertFalse(self._callFUT(parent, child))
        self.assertTrue(self._callFUT(child, child))
        self.assertTrue(self._callFUT(grand, parent))
        self.assertFalse(self._callFUT(parent, grand))
        self.assertTrue(self._callFUT(grand, child))
        self.assertFalse(self._callFUT(child, grand))
        self.assertTrue(self._callFUT(grand, grand))


class ClassAndInstanceDescrTests(unittest.TestCase):

    def _getTargetClass(self):
        from zope.location.location import ClassAndInstanceDescr
        return ClassAndInstanceDescr

    def _makeOne(self, _inst, _class):
        return self._getTargetClass()(_inst, _class)

    def _makeScaffold(self):
        _inst_called = []

        def _inst(*args, **kw):
            _inst_called.append((args, kw))
            return 'INST'
        _class_called = []

        def _class(*args, **kw):
            _class_called.append((args, kw))
            return 'CLASS'

        class Foo:
            descr = self._makeOne(_inst, _class)
        return Foo, _class_called, _inst_called

    def test_fetched_from_class(self):
        Foo, _class_called, _inst_called = self._makeScaffold()
        self.assertEqual(Foo.descr, 'CLASS')
        self.assertEqual(_class_called, [((Foo,), {})])
        self.assertEqual(_inst_called, [])

    def test_fetched_from_instance(self):
        Foo, _class_called, _inst_called = self._makeScaffold()
        foo = Foo()
        self.assertEqual(foo.descr, 'INST')
        self.assertEqual(_class_called, [])
        self.assertEqual(_inst_called, [((foo,), {})])


_MARKER = object()


class LocationProxyTests(unittest.TestCase, ConformsToILocation):

    def _getTargetClass(self):
        from zope.location.location import LocationProxy
        return LocationProxy

    def _makeOne(self, obj=None, container=_MARKER, name=_MARKER):
        if obj is None:
            obj = object()
        if container is _MARKER:
            self.assertIs(name, _MARKER)
            return self._getTargetClass()(obj)
        self.assertIsNot(name, _MARKER)
        return self._getTargetClass()(obj, container, name)

    def test_ctor_defaults(self):
        dummy = object()  # can't setattr
        proxy = self._makeOne(dummy)
        self.assertEqual(proxy.__parent__, None)
        self.assertEqual(proxy.__name__, None)

    def test_ctor_explicit(self):
        dummy = object()  # can't setattr
        parent = object()
        proxy = self._makeOne(dummy, parent, 'name')
        self.assertIs(proxy.__parent__, parent)
        self.assertEqual(proxy.__name__, 'name')

    def test___getattribute___wrapped(self):
        class Context:
            attr = 'ATTR'
        context = Context()
        proxy = self._makeOne(context)
        self.assertEqual(proxy.attr, 'ATTR')

    def test___setattr___wrapped(self):
        class Context:
            attr = 'BEFORE'
        context = Context()
        proxy = self._makeOne(context)
        proxy.attr = 'AFTER'
        self.assertEqual(context.attr, 'AFTER')

    def test___doc___from_derived_class(self):
        klass = self._getTargetClass()

        class Derived(klass):
            """DERIVED"""
        self.assertEqual(Derived.__doc__, 'DERIVED')

    def test___doc___from_target_class(self):
        class Context:
            """CONTEXT"""
        proxy = self._makeOne(Context())
        self.assertEqual(proxy.__doc__, 'CONTEXT')

    def test___doc___from_target_instance(self):
        class Context:
            """CONTEXT"""
        context = Context()
        context.__doc__ = 'INSTANCE'
        proxy = self._makeOne(context)
        self.assertEqual(proxy.__doc__, 'INSTANCE')

    def test___reduce__(self):
        proxy = self._makeOne()
        self.assertRaises(TypeError, proxy.__reduce__)

    def test___reduce_ex__(self):
        proxy = self._makeOne()
        self.assertRaises(TypeError, proxy.__reduce_ex__, 1)

    def test___reduce___via_pickling(self):
        import pickle

        class Context:
            def __reduce__(self):
                raise AssertionError("This is not called")
        proxy = self._makeOne(Context())
        # XXX: this TypeError is not due to LocationProxy.__reduce__:
        #      it's descriptor (under pure Python) isn't begin triggered
        #      properly
        self.assertRaises(TypeError, pickle.dumps, proxy)

    def test__providedBy___class(self):
        from zope.interface import Interface
        from zope.interface import implementer
        from zope.interface import providedBy
        from zope.interface import provider

        class IProxyFactory(Interface):
            pass

        class IProxy(Interface):
            pass

        @provider(IProxyFactory)
        @implementer(IProxy)
        class Foo(self._getTargetClass()):
            pass
        self.assertEqual(list(providedBy(Foo)), [IProxyFactory])

    def test__providedBy___instance(self):
        from zope.interface import Interface
        from zope.interface import implementer
        from zope.interface import providedBy
        from zope.interface import provider

        from zope.location.interfaces import ILocation

        class IProxyFactory(Interface):
            pass

        class IProxy(Interface):
            pass

        class IContextFactory(Interface):
            pass

        class IContext(Interface):
            pass

        @provider(IProxyFactory)
        @implementer(IProxy)
        class Proxy(self._getTargetClass()):
            pass

        @provider(IContextFactory)
        @implementer(IContext)
        class Context:
            pass
        context = Context()
        proxy = Proxy(context)
        self.assertEqual(list(providedBy(proxy)), [
                         IContext, IProxy, ILocation])


class LocationPyProxyTests(LocationProxyTests):

    def setUp(self):
        import sys
        for mod in ('zope.location.location',
                    'zope.proxy.decorator'):
            try:
                del sys.modules[mod]
            except KeyError:  # pragma: no cover
                pass
        import zope.proxy
        self.orig = (zope.proxy.ProxyBase,
                     zope.proxy.getProxiedObject,
                     zope.proxy.setProxiedObject,
                     zope.proxy.isProxy,
                     zope.proxy.sameProxiedObjects,
                     zope.proxy.queryProxy,
                     zope.proxy.queryInnerProxy,
                     zope.proxy.removeAllProxies,
                     zope.proxy.non_overridable)
        zope.proxy.ProxyBase = zope.proxy.PyProxyBase
        zope.proxy.getProxiedObject = zope.proxy.py_getProxiedObject
        zope.proxy.setProxiedObject = zope.proxy.py_setProxiedObject
        zope.proxy.isProxy = zope.proxy.py_isProxy
        zope.proxy.sameProxiedObjects = zope.proxy.py_sameProxiedObjects
        zope.proxy.queryProxy = zope.proxy.py_queryProxy
        zope.proxy.queryInnerProxy = zope.proxy.py_queryInnerProxy
        zope.proxy.removeAllProxies = zope.proxy.py_removeAllProxies
        zope.proxy.non_overridable = zope.proxy.PyNonOverridable

    def tearDown(self):
        import zope.proxy
        (zope.proxy.ProxyBase,
         zope.proxy.getProxiedObject,
         zope.proxy.setProxiedObject,
         zope.proxy.isProxy,
         zope.proxy.sameProxiedObjects,
         zope.proxy.queryProxy,
         zope.proxy.queryInnerProxy,
         zope.proxy.removeAllProxies,
         zope.proxy.non_overridable) = self.orig


def test_suite():
    return unittest.defaultTestLoader.loadTestsFromName(__name__)



##############################################################################
#
# Copyright (c) 2006 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
# This package is developed by the Zope Toolkit project, documented here:
# http://docs.zope.org/zopetoolkit
# When developing and releasing this package, please follow the documented
# Zope Toolkit policies as described by this documentation.
##############################################################################
"""Setup for zope.location package
"""
import os

from setuptools import find_packages
from setuptools import setup


def read(*rnames):
    with open(os.path.join(os.path.dirname(__file__), *rnames)) as f:
        return f.read()


setup(
    name='zope.location',
    version='5.1',
    author='Zope Foundation and Contributors',
    author_email='zope-dev@zope.dev',
    description='Zope Location',
    long_description=(
        read('README.rst') + '\n\n' + read('CHANGES.rst')),
    license='ZPL-2.1',
    keywords=('zope location structural'),
    classifiers=[
            'Development Status :: 5 - Production/Stable',
            'Environment :: Web Environment',
            'Intended Audience :: Developers',
            'License :: OSI Approved :: Zope Public License',
            'Programming Language :: Python',
            'Programming Language :: Python :: 3',
            'Programming Language :: Python :: 3.9',
            'Programming Language :: Python :: 3.10',
            'Programming Language :: Python :: 3.11',
            'Programming Language :: Python :: 3.12',
            'Programming Language :: Python :: 3.13',
            'Programming Language :: Python :: Implementation :: CPython',
            'Programming Language :: Python :: Implementation :: PyPy',
            'Natural Language :: English',
            'Operating System :: OS Independent',
            'Topic :: Internet :: WWW/HTTP',
            'Framework :: Zope :: 3',
            'Framework :: Zope :: 5',
    ],
    url='http://github.com/zopefoundation/zope.location/',
    packages=find_packages('src'),
    package_dir={
        '': 'src'},
    namespace_packages=[
        'zope',
    ],
    python_requires='>=3.9',
    install_requires=[
        'setuptools',
        'zope.interface>=4.0.2',
        'zope.schema>=4.2.2',
        'zope.proxy>=4.0.1',
    ],
    extras_require={
        'zcml': ['zope.configuration'],
        'component': ['zope.component >= 4.0.1'],
        'copy': ['zope.copy >= 4.0'],
        'test-minimal': ['zope.testrunner'],
        'test-component': [
            'zope.testrunner',
            'zope.component >= 4.0.1',
            'zope.configuration',
        ],
        'test': [
            'zope.testrunner',
            'zope.configuration',
            'zope.component >= 4.0.1',
            'zope.copy >= 4.0',
        ],
        'docs': [
            'repoze.sphinx.autointerface',
            'Sphinx',
            'zope.component >= 4.0.1',
            'zope.configuration',
        ],
    },
    include_package_data=True,
    zip_safe=False,
)



import importlib
import unittest


def skipUnlessImportable(module: str):
    try:
        importlib.import_module(module)
    except ModuleNotFoundError:  # pragma: no cover
        return unittest.skip(f"{module!r} not importable")
    return lambda func: func
